package org.cvo.fithas.dbsupport;

/** 
* Copyright @ 2004 Tianjin University IBM center. 
* All right reserved. 
*/ 

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import org.cvo.fithas.exception.DBOperationException;
import org.cvo.fithas.exception.DuplicateRecordException;
import org.cvo.fithas.exception.IllegalFileException;
import org.cvo.fithas.exception.IllegalPathException;
import org.cvo.fithas.exception.NotSpecifiedInfoException;
import org.cvo.fithas.exception.TableAlreadyExistException;
import org.cvo.fithas.exception.TableNotExistsException;
import org.cvo.fithas.fitsfile.GenericHeadAnalysis;
import org.cvo.fithas.ui.wizard.headerarchiving.ImportFileStatus;
import org.cvo.fithas.util.PathConfHelper;
import org.cvo.fithas.util.SystemConstant;
import org.cvo.fithas.util.Utils;

/** 
* The class is for dealing with the database.
*/ 
public class DbControl{
	
private static Connection theConnection = null; //singleton?
private static Statement theStatement = null; //singleton?

static org.cvo.fithas.util.Logger logger = new org.cvo.fithas.util.Logger(DbControl.class.getName () ) ;


/** Get previous opened connection
  */
public static Connection getConnection() 
{
	return theConnection;
}

/** 
 * get Connection, using the specified connection settings
 * Any previous open connection should be closed, to avoid mistaken modifying in previous connection
 * @param info 
 * 			DBConnectionInfo : contains server ip, port, username, password, database name, table name, etc.
 * @throws SQLException
 * @throws ClassNotFoundException 
 * @throws IllegalAccessException 
 * @throws InstantiationException 
 *  */
public static Connection getConnection(DBConnectionInfo info) 
	throws SQLException, ClassNotFoundException, InstantiationException, IllegalAccessException 
{
	
	//1st.رǰ
	try{
		//if not exists, 
		if(info==null)
			return null;
		//if exists, close the previous connection
		if(theConnection!=null)
		{
			synchronized (theConnection) {
				theConnection.close();
			}
		}
	}catch(SQLException e) //it's thrown only when there is connection operations involved
	{
		//TODO do not close the connection???
		DbControl.closeConn();
		logger.debug("SQLException:"+e.getMessage()+":\n\tCause is:"+e.getCause());
		throw e;
	}		
	
	//2nd.עdriver
		if(info.getDriver()==null || info.getDriver().equals(""))
			info.setDriver(SystemConstant.DB_DEFAULT_DRIVER_MYSQL); //constant. MySQL for default db
		switch(info.getDbType())
		{
		case 0:
			info.setDriver(SystemConstant.DB_DEFAULT_DRIVER_MYSQL);
			break;
		case 1:
			info.setDriver(SystemConstant.DB_DRIVER_ORACLE);
			break;
		case 2:
			info.setDriver(SystemConstant.DB_DRIVER_SQLSERVER);
			break;
		default:
			info.setDriver(SystemConstant.DB_DEFAULT_DRIVER_MYSQL);
			break;
				
		}
		Driver driver = (Driver) Class.forName(info.getDriver()).newInstance();
		DriverManager.registerDriver(driver);
	
		
	//3rd, 
	try{
		if (info.getUserName()!=null && info.getUserName().length() > 0) 
		{
			  logger.debug("Connecting to :"+info.getUrl()+" user="+info.getUserName()+"...");
		    theConnection = DriverManager.getConnection(info.getUrl(),
			                                   info.getUserName(),
			                                   info.getPassword());
			  
		}
		else {
			logger.debug("Connecting to :"+info.getUrl()+" (anonymous)...");
		    theConnection = DriverManager.getConnection(info.getUrl());
		    
		}
		logger.debug("Connected to MySQL Server.");
		
	} catch(SQLException e) //it's thrown only when there is connection operations involved
	{
		//TODO do not close the connection???
		DbControl.closeConn();
		logger.debug("SQLException:"+e.getMessage()+":\n\tCause:"+e.getCause());
		throw e;
	}
	return theConnection;
}

/**
 * Get the statement.
 * if connection/statement not established yet, setup first.
 * */
public static Statement getStatement() throws SQLException
{
	if(theStatement!=null)
		return theStatement;
	
	//create statment if there isn't any established
	if(theConnection!=null)
		theStatement = theConnection.createStatement();//20070417 modified
	logger.debug("***[DbControl][getStatement]*** (statement==null) is "+(theStatement==null));
	return theStatement; 
}



/** 20070419 modified
 * generate create table sql 
 * field "record_Id"(primary key) and "path"(relative path) will be the default column.
 * put each column's name inside `` , and value inside ''
 *  */
public static String composeCreateTableSQL(DBConnectionInfo info, List columnsList) 
{
	String sql ="";
	StringBuffer sb =  new StringBuffer();
	//TODO or return null??
	if(info==null)
		return "";
	if(info.getDbType()==0)
	{
		sb.append("create table ");
		sb.append(info.getTableName()+"(");
		//record id(primary key)
		sb.append("`").append(SystemConstant.DB_COLUMNNAME_RECORDID_MYSQL).append("` ")
		.append(SystemConstant.DB_DATATYPE_MYSQL_RECORDID).append(" NOT NULL primary key auto_increment"); //

		//relative path(varchar)
		sb.append(",`").append(SystemConstant.DB_COLUMNNAME_RELATIVEPATH_MYSQL).append("` ")
		.append(SystemConstant.DB_DATATYPE_MYSQL_RELATIVEPATH); //e.g: ",path text"
		if(SystemConstant.DB_COLUMNLENGTH_RELATIVEPATH != SystemConstant.DB_COLUMNLENGTH_UNKNOWN)//δֵ֪ɹʾ
		{
			sb.append("("+ SystemConstant.DB_COLUMNLENGTH_RELATIVEPATH +")"); //䳤ȵĶ
		}


		//other columns from fits' head
		for(Iterator it=columnsList.iterator(); it.hasNext();)
		{
			TableColumnBean tc = (TableColumnBean) it.next();//each element in the list is TableColumnBean 
			sb.append(", `"+tc.getColumnName()+"` "+tc.getType());
			if(tc.getLength()!=SystemConstant.DB_COLUMNLENGTH_UNKNOWN)//δ֪䳤ֵ
			{
				sb.append("("+tc.getLength()+")"); //䳤ȵĶ
			}
			sb.append(" COMMENT '"+tc.getComment()+"'");
		}

		//make sure that no duplicate file
		sb.append(", UNIQUE KEY `uniquefile` (`" +SystemConstant.DB_COLUMNNAME_RELATIVEPATH_MYSQL+"`("
				+SystemConstant.DB_COLUMNLENGTH_UNIQUEKEY+") ) "); //unique(full_path),with key length 255
		sb.append(") ENGINE=MyISAM DEFAULT CHARSET=utf8;");
	}
	else if (info.getDbType()==2)
	{
		sb.append("create table ");
		sb.append(info.getTableName()+"(");
		//record id(primary key)
		sb.append(SystemConstant.DB_COLUMNNAME_RECORDID_MYSQL).append(" ").append(SystemConstant.DB_DATATYPE_MYSQL_RECORDID).append(" NOT NULL primary key identity(1,1)"); //

		//relative path(varchar)
		sb.append(",").append(SystemConstant.DB_COLUMNNAME_RELATIVEPATH_MYSQL).append(" ")
		.append(SystemConstant.DB_DATATYPE_MYSQL_STRING).append("(255) unique not null"); //e.g: ",path text"
		if(SystemConstant.DB_COLUMNLENGTH_RELATIVEPATH != SystemConstant.DB_COLUMNLENGTH_UNKNOWN)//δֵ֪ɹʾ
		{
			sb.append("("+ SystemConstant.DB_COLUMNLENGTH_RELATIVEPATH +")"); //䳤ȵĶ
		}


		//other columns from fits' head
		for(Iterator it=columnsList.iterator(); it.hasNext();)
		{
			TableColumnBean tc = (TableColumnBean) it.next();//each element in the list is TableColumnBean 
			String columnName=tc.getColumnName();
			if(columnName.contains("-")||containsSepciedChar(columnName))
				columnName="["+columnName+"]";
			sb.append(", "+columnName+" "+tc.getType());
			if(tc.getLength()!=SystemConstant.DB_COLUMNLENGTH_UNKNOWN)//δ֪䳤ֵ
			{
				sb.append("("+tc.getLength()+")"); //䳤ȵĶ
			}
			//sb.append(" COMMENT '"+tc.getComment()+"'");
		}

		//make sure that no duplicate file
		sb.append(" ) "); //unique(full_path),with key length 255

		
	}
	sql = sb.toString();
	logger.debug("Create table sql="+sql);
	return sql;
}
static boolean containsSepciedChar(String column)
{
	boolean result=false;
	if(column!=null)
	{
		char temp=column.charAt(0);
		if(temp>='0'&&temp<='9')
			result=true;
		
	}
	return result;
}

/**
 * Create a table in db.
 * Connect to the target database, check table existence, and create it.
 * @throws SQLException 
 * */
public static String createTable(DBConnectionInfo info, List columnsList)
	throws TableAlreadyExistException, NotSpecifiedInfoException, DBOperationException, SQLException {

		Statement sm = null;
		String sql = null;
      
      	//check information completement
	  	if(info==null || info.getTableName()==null || info.getTableName().equals(""))
	  	{
	  		throw new NotSpecifiedInfoException(); //then skip parts below
	  	}
  	
        try {//TODO checkTableExists()쳣/ֵܷȷӳ
            //check if the table already exists
            boolean tableExists = checkTableExists(info); //should return boolean result
            if(tableExists)
            {
        		logger.debug("--------[DbControl][createTable]---The table:"+info.getTableName()+" already exists!");
        		throw new TableAlreadyExistException();//more info to show the cause of failure
            }
    		logger.debug("--------[DbControl][createTable]---after checkTableExists, the result is:"+tableExists);
    	           
            //continue to create the table
            sm = DbControl.getConnection(info).createStatement(); //the connection is opened in the check step above
            //turn to the specified database
            sm.addBatch("USE "+info.getDatabase()+";");
            //õݱSQL
            sql = composeCreateTableSQL(info, columnsList);
            sm.addBatch(sql);
            //ύִ
            sm.executeBatch();
            
        } catch (SQLException e) {
            throw new SQLException(e.getMessage());
        } catch (ClassNotFoundException e) {
			throw new DBOperationException(e.getMessage());
		} catch (InstantiationException e) {
            throw new DBOperationException(e.getMessage());
		} catch (IllegalAccessException e) {
            throw new DBOperationException(e.getMessage());
		} catch (NotSpecifiedInfoException e) {
			throw new NotSpecifiedInfoException();
		} finally { //ִйرղ
            DbControl.closeAll();
        }
		return sql;
 	}
 
/**
 * Query out the table columns description from db.
 * Can only recorgnize types like: 
 *    "XX(nnn)"--e.g: bigint(20), or: varchar(80).
 *    "text" --e.g: text --> and we know it's : text(65535) as we set the length of "text"=65535 when we create tables
 *    */
public static List queryTableDescription(DBConnectionInfo info)
	throws NotSpecifiedInfoException, SQLException, ClassNotFoundException, InstantiationException, IllegalAccessException
{
	Statement sm = null;
	ResultSet rs = null;
	String sql = "";
	List tableColumnDescList = new ArrayList();
	String columnName="", columnType="";
	TableColumnBean tcb;
	int tempIndex1=0, tempIndex2=0;
	
	if(info==null)
	{
		throw new NotSpecifiedInfoException(); //info is not specified properly
	}
	
	logger.debug("--------[DbControl][queryTableDescription]---checking table:"+info.getTableName()+" ...");
	theConnection = DbControl.getConnection(info);
    sm = theConnection.createStatement(); 
    sm.execute("USE "+info.getDatabase()+";");

    if(info.getDbType()==SystemConstant.DB_TYPE_MYSQL)//mysql
    	 sql = "SHOW full COLUMNS FROM "+info.getTableName(); //
    else if(info.getDbType()==SystemConstant.DB_TYPE_ORACLE)
    	sql = "desc "+info.getTableName();
    else if(info.getDbType()==SystemConstant.DB_TYPE_SQLSERVER)
    	sql = "select left(A.name,   20),left(C.name,20),A.length from syscolumns A left join  sysobjects B  on   A.id   =   B.id   left   join   systypes   C  on   A.xtype   =   C.xtype where   B.name   =   '"+info.getTableName()+"';";
    
    try{
	    rs = sm.executeQuery(sql);
		logger.debug("--------[DbControl][queryTableDescription]---after query from table:"+info.getTableName()+":");
	    if(rs!=null)
	    {//the only situation that means test finished, and table does exist
			logger.debug("--------[DbControl][queryTableDescription]---we got the table description:");
			while(rs.next())
			{
				
				columnName = (String)rs.getString(1); //should use "1"
				tcb = new TableColumnBean();
				//column name
				tcb.setColumnName(columnName);
				//column type and length
				columnType = (String)rs.getString(2); //should use "2"
				if(columnType.indexOf("(")>0 && columnType.indexOf(")")>0)
				{
					tempIndex1 = columnType.indexOf("(");
					tcb.setType(columnType.substring(0,tempIndex1));
					tempIndex2 = columnType.indexOf(")");
					tcb.setLength(Integer.parseInt(columnType.substring(tempIndex1+1, tempIndex2)));
				}
				else //if(columnType.equalsIgnoreCase(SystemConstant.DB_DATATYPE_MYSQL_VLONGSTRING))//"text" or "mediumtext"
				{
					tcb.setType(columnType);
					if(info.getDbType()==SystemConstant.DB_TYPE_SQLSERVER)
						tcb.setLength(rs.getInt(3));
					else
					tcb.setLength(SystemConstant.DB_COLUMNLENGTH_UNKNOWN);//.DB_COLUMNLENGTH_TEXT_LONG);//65535(max length of type "text" in mysql)
				}
				//column comment
				if(info.getDbType()==0)
				{
					if(rs.getString(9)!=null)
					{
						tcb.setComment((String)rs.getString(9));
					}

					
				}
				tableColumnDescList.add(tcb);
//				logger.debug("column :"+columnName+" "+columnType);				
			}
	    }
    } catch(SQLException e) //table not exists, etc
    {
    	System.err.println("--[DbControl][queryTableDescription]--SQLException: "+e.getErrorCode()+" : "+e.getMessage());
		throw new SQLException(e.getMessage());
    }

    return tableColumnDescList; 
}

/**
 * Query tables(table name) from the given db. 
 * @return List -- the list of table names, can be used to help user select a table to import to
 * */
public static List getTables(DBConnectionInfo info)
	throws NotSpecifiedInfoException, SQLException, ClassNotFoundException, InstantiationException, IllegalAccessException
{
	Statement sm = null;
	ResultSet rs = null;
    String sql = "";
	List availableTables = new ArrayList();
	
	if(info==null)
	{
		throw new NotSpecifiedInfoException(); //info is not specified properly
	}
	
	logger.debug("--------[DbControl][checkTableExists]---checking table:"+info.getTableName()+" ...");
		theConnection = DbControl.getConnection(info);
	    sm = theConnection.createStatement(); 
	    sm.execute("USE "+info.getDatabase()+";");

	    if(info.getDbType()==SystemConstant.DB_TYPE_MYSQL)//mysql
	    	 sql = "show tables;"; //
	    else if(info.getDbType()==SystemConstant.DB_TYPE_ORACLE)
	    	sql = "select * from tab;";
	    else if(info.getDbType()==SystemConstant.DB_TYPE_SQLSERVER)
	    	sql = "select TABLE_NAME from information_schema.tables;";
	    
	    try{
		    rs = sm.executeQuery(sql);
			logger.debug("--------[DbControl][getAvailableTables]---after query from table:"+info.getTableName()+":");
		    if(rs!=null)
		    {//the only situation that means test finished, and table does exist
				logger.debug("--------[DbControl][getAvailableTables]---we got the tables:");
				while(rs.next())
				{
					String table = (String)rs.getString(1); //should use "1"
			    	availableTables.add(table);
				}
		    }
	    } catch(SQLException e) //table not exists, etc
	    {
	    	System.err.println("--[DbControl][getAvailableTables]--SQLException: "+e.getErrorCode()+" : "+e.getMessage());
    		throw new SQLException(e.getMessage());
	    }
	
	return availableTables; 
}

/**
 * Check whether the specified table already exist, by means of query from it.
 * if any exception thrown, return false as default, caller should catch exceptions and take actions 
 * 
 * @return boolean: true if the table exists, false otherwise(including failure during check)
 * 
 * @throws NotSpecifiedInfoException 
 * @throws IllegalAccessException 
 * @throws InstantiationException 
 * @throws ClassNotFoundException 
 * @throws SQLException 
 * */
public static boolean checkTableExists(DBConnectionInfo info) 
	throws NotSpecifiedInfoException, ClassNotFoundException, InstantiationException, IllegalAccessException, SQLException 
	 {
	
	Statement sm = null;
	ResultSet rs = null;
	boolean exists = false;
	if(info==null)
	{
		throw new NotSpecifiedInfoException(); //info is not specified properly
	}
	
	logger.debug("--------[DbControl][checkTableExists]---checking table:"+info.getTableName()+" ...");
		theConnection = DbControl.getConnection(info);
	    sm = theConnection.createStatement(); 
	    sm.execute("USE "+info.getDatabase()+";");
	    String sql = "select * from "+info.getTableName();

	    try{
		    rs = sm.executeQuery(sql); //if the table not exists, exception will be thrown
			logger.debug("--------[DbControl][checkTableExists]---after query from table:"+info.getTableName()+":");
		    if(rs!=null)
		    {//the only situation that means test finished, and table does exist
				logger.debug("--------[DbControl][checkTableExists]---The table '"+info.getTableName()+"' exists!");
		    	exists = true; //does exist
		    }
	    } catch(SQLException e) //table not exists, etc
	    {
	    	logger.debug("--[DbControl][checkTableExists]--SQLException: "+e.getMessage());
	    	if(e.getErrorCode()==1146||e.getErrorCode()==208) //1146: General error message from server: "Table 'test.bais_test' doesn't exist"
	    		return false;//not exists
	    	else 
	    	{//cannot infer that the table exists or not!
	    		throw e;
	    	}
	    } finally
	    {
	    	DbControl.closeAll();
	    }
	
	return exists; 
}

	/**
	 *  closeConn(): close the connection  
	 * @throws SQLException 
	 *  */
	public static void closeConn() throws SQLException
	{
		if(theConnection!=null)
		{
			synchronized (theConnection){
				theConnection.close();
				theConnection = null;
			}
		}		
	}
	
	/**
	 * Close the statement.
	 * */
	public static void closeStatement() throws SQLException
	{
		if(theStatement!=null)
		{
			synchronized (theStatement) {
				theStatement.close();
				theStatement = null;
			}
		}
	}
	/**
	 * Close statement and connection.
	 * */
	public static void closeAll() throws SQLException
	{
		closeStatement();
		closeConn();	
	}

	/**
	 * Test connection with the specified database, without testing the table existence.
	 * @throws DBOperationException 
	 * @throws SQLException 
	 * */
	public static boolean testConnection(DBConnectionInfo connInfo) 
							throws NotSpecifiedInfoException, DBOperationException, SQLException {
	
		boolean isSuccess = false;
	
		if(connInfo==null)
		{
			throw new NotSpecifiedInfoException(); //
		}
		
		try {
			//close previous connection and statement(if exists)	
			DbControl.closeAll();
			
			//make the connection using the given info
			theConnection = DbControl.getConnection(connInfo);
			if(theConnection!=null)
			{
				theStatement = theConnection.createStatement(); //
				// ǰתconnInfo.getDatabase()
				if(theStatement!=null)
				{
					theStatement.execute("USE "+connInfo.getDatabase()+";");
					isSuccess = true;
				}
				else //can't create statement
				{
					isSuccess = false;
				}
			}
			else //can't get connection
			{
				isSuccess = false;
			}
            
		} catch (ClassNotFoundException e) {
			throw new DBOperationException(e.getMessage());
		} catch (InstantiationException e) {
			throw new DBOperationException(e.getMessage());
		} catch (IllegalAccessException e) {
			throw new DBOperationException(e.getMessage());
		} 
		finally
		{
			DbControl.closeAll();
		}
	
   
	return isSuccess;
}

	public static void main(String[] args)
	{
		
//		String driverName = "com.microsoft.jdbc.sqlserver.SQLServerDriver"; 
//		String dbURL = "jdbc:microsoft:sqlserver://localhost:1433; DatabaseName=pubs"; 
//		String userName = "sa"; 
//		String userPwd = "sa"; 
//		Connection dbConn;
//
//		try { 
//		Class.forName(driverName); 
//		dbConn = DriverManager.getConnection(dbURL, userName, userPwd); 
//		System.out.println("Connection Successful!"); 
//		}catch (Exception e) { 
//		e.printStackTrace(); 
//		} 

		Connection theConnection=null;
		Statement theStatement=null;
		try
		{
			//Class.forName("com.microsoft.jdbc.sqlserver.SQLServerDriver")
		Driver driver = (Driver) Class.forName("com.microsoft.jdbc.sqlserver.SQLServerDriver").newInstance();
		DriverManager.registerDriver(driver);
		theConnection = DriverManager.getConnection("jdbc:microsoft:sqlserver://localhost:1433; DatabaseName=pubs",
			                                   "sa",
			                                   "sa");
		theStatement = theConnection.createStatement();
		if(theStatement!=null)
		{
					theStatement.execute("USE sdss;");
		}
		}catch(Exception exe)
		{
			exe.printStackTrace();
		}
//		try
//		{
//			Class.forName("com.mysql.jdbc.Driver"); 
//			String url="jdbc:mysql://localhost:3306/test";
//			theConnection=DriverManager.getConnection(url,"root","tester");
//			theConnection.setAutoCommit(false);
//			String sql="insert into cc(record_Id,path,SIMPLE,BITPIX) values(?,?,?,?)";
//			theStatement = theConnection.prepareStatement(sql);
//
//			long time1=System.currentTimeMillis();
//			for(int i=0;i<20000;i++)
//			{
//				theStatement.setInt(1, i);
//				theStatement.setString(2, i+".txt");
//				theStatement.setString(3,"SCI_SN");
//				theStatement.setString(4, "XCHI2");
//
//				//theStatement.executeUpdate(sql);
//				theStatement.addBatch();
//			}
//			theStatement.executeBatch();
//			theConnection.commit();
//			long time2=System.currentTimeMillis();
//			System.out.println(time2-time1);
//		}catch(Exception exe)
//		{
//			exe.printStackTrace();
//		}finally
//		{
//			try {
//				theStatement.close();
//			} catch (SQLException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
//			try {
//				theConnection.close();
//			} catch (SQLException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
//		}
	}




		/** <P>Import a single file to DB. Mainly used in headerArchiving(single file).</P>
		 * <P>3 steps:</P>
		 * <LI>		1.connect</LI>
		 * <LI> 	2.call progress --(recursively) analysis file(s) and do import</LI>
		 * <LI>		3.disconnect</LI> 
		 * <pre>Ϊͳԣʹ֮׶ͬʱΪõʾļĳɹʧϢ
		 * ؽ"ظļ¼"µ"ص"ʽ߼ڸ÷ʵ֣ҲٵһrecursiveFileImport()
		 * ظļ¼recursiveFileImport()׳DuplicateRecordException쳣catchסУ
		 * 	(1)table(2)ٴεrecursiveFileImport() ,
		 * £tableٴεrecursiveFileImport()ӦòDuplicateRecordException
		 * </pre>
		 * @param info
		 * @param datafilePath
		 * @param successFileInfo 
		 * 			Hashtable --ϷɹļϢ
		 * @param failedFileInfo
		 * 			Hashtable --ϷʧļϢ
		 * 
		 * @throws NotSpecifiedInfoException
		 * @throws DBOperationException
		 * @throws TableNotExistsException
		 * @throws SQLException
		 * @throws IllegalPathException
		 * @throws DuplicateRecordException 
		 */
		public static void importToDb(DBConnectionInfo info, String datafilePath, String filter, int duplicateMethodType, ArrayList successFileInfo, ArrayList failedFileInfo)
						throws DBOperationException, TableNotExistsException, SQLException, IllegalPathException, DuplicateRecordException {

			logger.debug("--------[DbControl][importToDb]--Begin to call recursive file analysis progress.");
			File file = null;
			
			try {
				file = new File(datafilePath);
				
				//open a (reusable) connection
				theConnection = DbControl.getConnection(info);
				theStatement = theConnection.createStatement();//reuse the statement
				logger.debug("----Got connection.Begin to call import");
				
				/* 20070417 modified, enable duplicate record management.*/
				if(duplicateMethodType==0)//clear table and rearchive
				{					
					doClearToRearchive(info, theStatement); //ָtable
				}
				//elseβر
				
				//recursiveFileImport()	
				recursiveFileImport(info, file, filter, duplicateMethodType, successFileInfo, failedFileInfo);
				//theStatement.executeBatch();
				
			} catch (ClassNotFoundException e) {
				throw new DBOperationException(e.getMessage());
			} catch (InstantiationException e) {
				throw new DBOperationException(e.getMessage());
			} catch (IllegalAccessException e) {
				throw new DBOperationException(e.getMessage());
			} 
			finally
			{
				DbControl.closeAll();
			}
			
		}


		/** 20070418 modified, enable duplicate record management.
		 * <P>For multi files import. Mainly used in batchArchiving(multi files/directory).</P>
		 * Almost same as importToDb(DBConnectionInfo, String, Hashtable, Hashtable)
		 * 
		 * @param info
		 * @param datafiles
		 * @param successFileInfo
		 * @param failedFileInfo
		 * 
		 * @throws NotSpecifiedInfoException
		 * @throws DBOperationException
		 * @throws TableNotExistsException
		 * @throws SQLException
		 * @throws IllegalPathException
		 * @throws DuplicateRecordException 
		 */
		public static void importToDb(DBConnectionInfo info, String[] datafiles, String filter, int duplicateMethodType, ArrayList successFileInfo, ArrayList failedFileInfo) 
				throws DBOperationException, TableNotExistsException, SQLException, IllegalPathException, DuplicateRecordException 
		{
			logger.debug("--------[DbControl][importToDb]--Begin to call recursive file analysis progress.");
			
			try {
				//open a (reusable) connection
				theConnection = DbControl.getConnection(info);
				theStatement = theConnection.createStatement();//reuse the statement
				logger.debug("----Got connection.Begin to call import");
				
				//clear table and rearchive
				if(duplicateMethodType==0)
				{					
					doClearToRearchive(info, theStatement); //ָtable
				}
				
				//then do import, and carry back success/failed info
				System.out.println(System.currentTimeMillis());
				
				if(datafiles!=null)
				{
					for(int i=0; i<datafiles.length; i++)
					{
						File file = new File(datafiles[i]); //for each element in array
						recursiveFileImport(info, file, filter, duplicateMethodType, successFileInfo, failedFileInfo);
					}
				}
				
				//theStatement.executeBatch();
				
				System.out.println(System.currentTimeMillis());
				
			} catch (ClassNotFoundException e) {
				throw new DBOperationException(e.getMessage());
			} catch (InstantiationException e) {
				throw new DBOperationException(e.getMessage());
			} catch (IllegalAccessException e) {
				throw new DBOperationException(e.getMessage());
			} 
			finally
			{
				DbControl.closeAll();
			}
		}
		
		
		public static void importToDb(DBConnectionInfo info, String[] datafiles, String filter, int duplicateMethodType, ArrayList successFileInfo, ArrayList failedFileInfo,String filterFileName,String contains) 
		throws DBOperationException, TableNotExistsException, SQLException, IllegalPathException, DuplicateRecordException 
{
	logger.debug("--------[DbControl][importToDb]--Begin to call recursive file analysis progress.");
	
	try {
		//open a (reusable) connection
		theConnection = DbControl.getConnection(info);
		theStatement = theConnection.createStatement();//reuse the statement
		logger.debug("----Got connection.Begin to call import");
		
		//clear table and rearchive
		if(duplicateMethodType==0)
		{					
			doClearToRearchive(info, theStatement); //ָtable
		}
		
		System.out.println(System.currentTimeMillis());
		//then do import, and carry back success/failed info
		if(datafiles!=null)
		{
			for(int i=0; i<datafiles.length; i++)
			{
				File file = new File(datafiles[i]); //for each element in array
				recursiveFileImport(info, file, filter, duplicateMethodType, successFileInfo, failedFileInfo,filterFileName,contains);
			}
		}
		System.out.println(System.currentTimeMillis());
		
	} catch (ClassNotFoundException e) {
		throw new DBOperationException(e.getMessage());
	} catch (InstantiationException e) {
		throw new DBOperationException(e.getMessage());
	} catch (IllegalAccessException e) {
		throw new DBOperationException(e.getMessage());
	} 
	finally
	{
		DbControl.closeAll();
	}
}
			

	/**
	 * <pre>ļݲݿĵݹ㷨
	 * ظ¼3ִʽʹ"ص"ʽձִ֮sqlʷбsqlHistory
	 * ִв
	 * 	 1.Ŀ¼еÿһļ/Ŀ¼ݹô˷
	 * 	 2.ļ
	 * 		(1)ýļheaderķ
	 * 		(2)ļ·Ϣ
	 * 		(3)Ȼsql
	 * 		(4)ִвsql¼
	 * Ⱦ
	 * 	 1.ѻҪӵݿϢ(DBConnectionInfo);
	 * 	 2.ѻݿ(theConnectiontheStatement);
	 * עﲻݿӡرգҲҪļǷfitsfits.gz
	 * </pre>
	 * @param sqlHistory 
	 * @throws DuplicateRecordException 
	 * */	
	private static void recursiveFileImport(DBConnectionInfo info, File f, String filter, int duplicateMethodType, ArrayList successInfoHt, ArrayList failedInfoHt) throws IllegalPathException, DuplicateRecordException 
	{
		logger.debug("---ENTER:[DbControl][recursiveFileImport],the file is:"+f+"--\n");
		
		//for directory, call recursively 
		logger.debug("f.exists()="+f.exists()+"; f.isDirectory()="+f.isDirectory()+"; f.isFile()="+f.isFile());
		if(f.isDirectory()) 
		{
			File[] file = f.listFiles();
			for(int i = 0; i<file.length; i++)
			{
				recursiveFileImport(info, file[i], filter, duplicateMethodType, successInfoHt, failedInfoHt);
			}
		}
		else if(f.isFile())
		{
			Hashtable htable = new Hashtable();
			try{		  		
				//judge file path(as 2 columns in table)
	    		String fullPath = f.getCanonicalPath().replace('\\','/');//see xuzhen's NameAnalysis.java
	            String basedir = PathConfHelper.getAttribute(SystemConstant.CONFIG_PATH_PROPFILE_KEY_BASEDIR);
	            String relativePath;//·
	      
	            //get info from file's head partHashtable
	            //жǷλbasedir
		        if(fullPath.indexOf(basedir)==0) //the file is under basedir
		        {
		            //ټļǷָpattern(filter)
			        if(Utils.checkFileNamePattern(fullPath, filter))//filterֻⲽõ
			        {
			            //ϵĲŽнļ
			        	logger.debug("The file passed pattern check.");
		            	htable = GenericHeadAnalysis.getFitsHeadHT(fullPath);
		            	//ټļ·һ
		            	relativePath = fullPath.substring(fullPath.indexOf(basedir)+basedir.length());
			            htable.put(SystemConstant.DB_COLUMNNAME_RELATIVEPATH_MYSQL, relativePath);

						//generate sql
						String sql = composeInsertSQL(info.getTableName(), htable,info);
						logger.debug("Composed sql is:---" +sql);
							
						//use(reuse)the statement to execute insert sql
						if(theStatement==null)//һ㲻ߴ˷֧
						{
							logger.debug("--null statement here!!!");
							theStatement = DbControl.getConnection(info).createStatement();
						}
			
						//execute sql to insert this file
						theStatement.executeUpdate(sql);
						//theStatement.addBatch(sql);
						//record the success info
						
						ImportFileStatus status=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_SUCCESS_MSG,SystemConstant.MSG_IMPORT_SUCCESS);
						successInfoHt.add(status);
						//successInfoHt.put(f.getPath(), SystemConstant.MSG_IMPORT_SUCCESS);
						
		            }//if ļpattern
			        //else patternļ

	            }//if ļλbasedir
	            else//Ҫļ/Ŀ¼basedir
	            {
	                throw new IllegalPathException();
	            }
							    
			} catch(IllegalPathException e){
				String msg;
				if(e==null || e.getMessage()==null)
				{
					msg = "IllegalPathException: File may be not under the base directory.";
				}
				else
				{
					msg = e.getMessage();
				}
				ImportFileStatus status=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,msg);
				
				failedInfoHt.add(status); // record the failed file's name
				
			} catch(SQLException e){
				if(e.getErrorCode()==1062) //duplicate key or integrity constraint violation 
				{
					switch(duplicateMethodType)//int
					{
						case SystemConstant.DB_ACTION_DUPLICATE_CLEARTABLE: //"ص"ʽ
							/* ΪݱֵظѲܱĬϹ򣺼¼Ϣskip */
							logger.debug("Duplicate action : clear table -- begin to throw exception back.");
							
							//failedInfoHt.put(f.getPath(), e.getErrorCode()+":"+e.getMessage());
							ImportFileStatus status=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getErrorCode()+":"+e.getMessage());
							
							failedInfoHt.add(status);
							throw new DuplicateRecordException();
//							break;
								
						case SystemConstant.DB_ACTION_DUPLICATE_SKIP: //"ظ¼"ʽ
							//¼ʧļϢjust skip
							//failedInfoHt.put(f.getPath(), e.getErrorCode()+":"+e.getMessage());
							ImportFileStatus status1=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getErrorCode()+":"+e.getMessage());
							
							failedInfoHt.add(status1);
							logger.debug("Duplicate action : skip -- record failed file and skip:"+f.getPath());
							break;
							
						case SystemConstant.DB_ACTION_DUPLICATE_UPDATE: //"ظ¼"ʽ
							try {
								//Ը¼
								doUpdateDuplicate(theStatement, info, htable);
								//³ɹļɹ
								ImportFileStatus status2=new ImportFileStatus(f.getPath(),SystemConstant.MSG_UPDATE_SUCCESS,SystemConstant.MSG_UPDATE_SUCCESS);
								 //successInfoHt.put(f.getPath(), SystemConstant.MSG_UPDATE_SUCCESS);
								successInfoHt.add(status2);
								 logger.debug("Duplicate action : update -- Updated:"+f.getPath());
							} catch (SQLException e1) {
								//ҲɣǾļʧ
								//failedInfoHt.put(f.getPath(), e.getErrorCode()+":"+e.getMessage());
								ImportFileStatus status3=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getErrorCode()+":"+e.getMessage());
								
								failedInfoHt.add(status3);
								logger.debug("Duplicate action : update -- Failed to update:"+f.getPath());
							}
							break;
							
						default :
							//¼ʧļϢjust skip
							ImportFileStatus status4=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getErrorCode()+":"+e.getMessage());
						
						    failedInfoHt.add(status4);
							//failedInfoHt.put(f.getPath(), e.getErrorCode()+":"+e.getMessage());
							logger.debug("Duplicate action : default action(skip) -- error message="+e.getMessage());
							
					} //switchduplicate¼3ִʽclearTable/skip/update
				} //if(ǳduplicate¼)
				
				//else,SQLException
				else{
					ImportFileStatus status5=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getErrorCode()+":"+e.getMessage());
					
					failedInfoHt.add(status5);
					
					//failedInfoHt.put(f.getPath(), e.getErrorCode()+":"+e.getMessage()); // record the failed file's name
				}
				
			} catch (InstantiationException e) {
				ImportFileStatus status6=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getMessage());
				
				failedInfoHt.add(status6);
			} catch (IllegalAccessException e) {
				ImportFileStatus status7=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getMessage());
				
				failedInfoHt.add(status7);
			} catch (ClassNotFoundException e) {
				ImportFileStatus status8=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getMessage());
				
				failedInfoHt.add(status8);
			} catch(IOException e){
				ImportFileStatus status9=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getMessage());
				
				failedInfoHt.add(status9);
			} catch (IllegalFileException e) {
				//do nothing,just skip the file
			}
		} //else if(f.isFile())
		//elseʹҲ
		
		logger.debug("---EXIT:[DbControl][recursiveFileImport]");
	}
	
	 public static boolean FilterFileName(String filename,String filter,String contains)
	    {
	    	
		 if(filter==null||filter.equals(""))
	    		return true;
	    	
	    	String temp=filter;
	    	String temp2="";
	    	int index=temp.lastIndexOf(".");
	    	if(index>0)
	    		temp2=temp.substring(0,index);
	    	else
	    		temp2=temp;
	    	String []temp3=temp2.split("\\*");
	    	boolean addfile=true;
	    	boolean isnotcontains=false;
	    	if(temp3!=null&&temp3.length>0)
	    	{

	    		for(int j=0;j<temp3.length;j++)
	    		{
	    			if(contains.equals(SystemConstant.REG_MATCH_PATTERNS[0]))
	    			{
	    				if(filename.contains(temp3[j]))
	    				{
	    					return true;
	    				}
	    			}
	    			else if(contains.equals(SystemConstant.REG_MATCH_PATTERNS[1]))
	    			{
	    				if(filename.startsWith(temp3[j]))
	    				{
	    					return true;
	    				}
	    			}
	    			else if(contains.equals(SystemConstant.REG_MATCH_PATTERNS[2]))
	    			{
	    				if(filename.endsWith(temp3[j]))
	    					return true;
	    			}
	    			else if(contains.equals(SystemConstant.REG_MATCH_PATTERNS[3]))
	    			{
	    				isnotcontains=true;
	    				if(filename.endsWith(temp3[j]))
	    					addfile=false;
	    			}
	    			else if(contains.equals(SystemConstant.REG_MATCH_PATTERNS[4]))
	    			{
	    				if(filename.equals(temp3[j]))
	    					return true;
	    			}
	    		}
	    	}
	    	if(isnotcontains)
	    		return addfile;
	    	else
	    		return false;
	    		
	    	
	    	
	    }
	
	private static void recursiveFileImport(DBConnectionInfo info, File f, String filter, int duplicateMethodType, ArrayList successInfoHt, ArrayList failedInfoHt,String filterFileName,String contains) throws IllegalPathException, DuplicateRecordException 
	{
		logger.debug("---ENTER:[DbControl][recursiveFileImport],the file is:"+f+"--\n");
		
		//for directory, call recursively 
		logger.debug("f.exists()="+f.exists()+"; f.isDirectory()="+f.isDirectory()+"; f.isFile()="+f.isFile());
		if(f.isDirectory()) 
		{
			File[] file = f.listFiles();
			for(int i = 0; i<file.length; i++)
			{
				recursiveFileImport(info, file[i], filter, duplicateMethodType, successInfoHt, failedInfoHt,filterFileName,contains);
			}
		}
		else if(f.isFile())
		{
			Hashtable htable = new Hashtable();
			try{		  		
				//judge file path(as 2 columns in table)
	    		String fullPath = f.getCanonicalPath().replace('\\','/');//see xuzhen's NameAnalysis.java
	            String basedir = PathConfHelper.getAttribute(SystemConstant.CONFIG_PATH_PROPFILE_KEY_BASEDIR);
	            String relativePath;//·
	      
	            //get info from file's head partHashtable
	            //жǷλbasedir
		        if(fullPath.indexOf(basedir)==0) //the file is under basedir
		        {
		            //ټļǷָpattern(filter)
			        if(Utils.checkFileNamePattern(fullPath, filter))//filterֻⲽõ
			        {
			        	if(FilterFileName(fullPath,filterFileName,contains))
			        	{
			        		//ϵĲŽнļ
			        		logger.debug("The file passed pattern check.");
			        		htable = GenericHeadAnalysis.getFitsHeadHT(fullPath);
			        		//ټļ·һ
			        		relativePath = fullPath.substring(fullPath.indexOf(basedir)+basedir.length());
			        		htable.put(SystemConstant.DB_COLUMNNAME_RELATIVEPATH_MYSQL, relativePath);

			        		//generate sql
			        		String sql = composeInsertSQL(info.getTableName(), htable,info);
			        		logger.debug("Composed sql is:---" +sql);

			        		//use(reuse)the statement to execute insert sql
			        		if(theStatement==null)//һ㲻ߴ˷֧
			        		{
			        			logger.debug("--null statement here!!!");
			        			theStatement = DbControl.getConnection(info).createStatement();
			        		}

			        		//execute sql to insert this file
			        		theStatement.executeUpdate(sql);
			        		//record the success info

			        		ImportFileStatus status=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_SUCCESS_MSG,SystemConstant.MSG_IMPORT_SUCCESS);
			        		successInfoHt.add(status);
			        	}
						//successInfoHt.put(f.getPath(), SystemConstant.MSG_IMPORT_SUCCESS);
						
		            }//if ļpattern
			        //else patternļ

	            }//if ļλbasedir
	            else//Ҫļ/Ŀ¼basedir
	            {
	                throw new IllegalPathException();
	            }
							    
			} catch(IllegalPathException e){
				String msg;
				if(e==null || e.getMessage()==null)
				{
					msg = "IllegalPathException: File may be not under the base directory.";
				}
				else
				{
					msg = e.getMessage();
				}
				ImportFileStatus status=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,msg);
				
				failedInfoHt.add(status); // record the failed file's name
				
			} catch(SQLException e){
				if(e.getErrorCode()==1062) //duplicate key or integrity constraint violation 
				{
					switch(duplicateMethodType)//int
					{
						case SystemConstant.DB_ACTION_DUPLICATE_CLEARTABLE: //"ص"ʽ
							/* ΪݱֵظѲܱĬϹ򣺼¼Ϣskip */
							logger.debug("Duplicate action : clear table -- begin to throw exception back.");
							
							//failedInfoHt.put(f.getPath(), e.getErrorCode()+":"+e.getMessage());
							ImportFileStatus status=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getErrorCode()+":"+e.getMessage());
							
							failedInfoHt.add(status);
							throw new DuplicateRecordException();
//							break;
								
						case SystemConstant.DB_ACTION_DUPLICATE_SKIP: //"ظ¼"ʽ
							//¼ʧļϢjust skip
							//failedInfoHt.put(f.getPath(), e.getErrorCode()+":"+e.getMessage());
							ImportFileStatus status1=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getErrorCode()+":"+e.getMessage());
							
							failedInfoHt.add(status1);
							logger.debug("Duplicate action : skip -- record failed file and skip:"+f.getPath());
							break;
							
						case SystemConstant.DB_ACTION_DUPLICATE_UPDATE: //"ظ¼"ʽ
							try {
								//Ը¼
								doUpdateDuplicate(theStatement, info, htable);
								//³ɹļɹ
								ImportFileStatus status2=new ImportFileStatus(f.getPath(),SystemConstant.MSG_UPDATE_SUCCESS,SystemConstant.MSG_UPDATE_SUCCESS);
								 //successInfoHt.put(f.getPath(), SystemConstant.MSG_UPDATE_SUCCESS);
								successInfoHt.add(status2);
								 logger.debug("Duplicate action : update -- Updated:"+f.getPath());
							} catch (SQLException e1) {
								//ҲɣǾļʧ
								//failedInfoHt.put(f.getPath(), e.getErrorCode()+":"+e.getMessage());
								ImportFileStatus status3=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getErrorCode()+":"+e.getMessage());
								
								failedInfoHt.add(status3);
								logger.debug("Duplicate action : update -- Failed to update:"+f.getPath());
							}
							break;
							
						default :
							//¼ʧļϢjust skip
							ImportFileStatus status4=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getErrorCode()+":"+e.getMessage());
						
						    failedInfoHt.add(status4);
							//failedInfoHt.put(f.getPath(), e.getErrorCode()+":"+e.getMessage());
							logger.debug("Duplicate action : default action(skip) -- error message="+e.getMessage());
							
					} //switchduplicate¼3ִʽclearTable/skip/update
				} //if(ǳduplicate¼)
				
				//else,SQLException
				else{
					ImportFileStatus status5=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getErrorCode()+":"+e.getMessage());
					
					failedInfoHt.add(status5);
					
					//failedInfoHt.put(f.getPath(), e.getErrorCode()+":"+e.getMessage()); // record the failed file's name
				}
				
			} catch (InstantiationException e) {
				ImportFileStatus status6=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getMessage());
				
				failedInfoHt.add(status6);
			} catch (IllegalAccessException e) {
				ImportFileStatus status7=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getMessage());
				
				failedInfoHt.add(status7);
			} catch (ClassNotFoundException e) {
				ImportFileStatus status8=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getMessage());
				
				failedInfoHt.add(status8);
			} catch(IOException e){
				ImportFileStatus status9=new ImportFileStatus(f.getPath(),SystemConstant.MSG_IMPORT_FAIL_MSG,e.getMessage());
				
				failedInfoHt.add(status9);
			} catch (IllegalFileException e) {
				//do nothing,just skip the file
			}
		} //else if(f.isFile())
		//elseʹҲ
		
		logger.debug("---EXIT:[DbControl][recursiveFileImport]");
	}
	
	
	/** "ظ¼"Ĵʽupdatesql䲢ִи
	 * @param theStatement
	 * @param info
	 * @param headerHt
	 * 			Hashtable --basedirfull_pathֶ
	 * @throws SQLException 
	 */
	private static void doUpdateDuplicate(Statement theStatement, DBConnectionInfo info, Hashtable headerHt) throws SQLException {
		String updateSql = composeUpdateSql(info, headerHt);
		theStatement.execute(updateSql);
	}

	/**
	 * ļϢupdatesql䡣 
	 * @param info
	 * @param headerHt
	 * @return
	 */
	private static String composeUpdateSql(DBConnectionInfo info, Hashtable headerHt) 
	{
		StringBuffer newValueString = new StringBuffer();
		StringBuffer sql = new StringBuffer();
		String key,value;
		
		//construct sql from the fields
		Iterator it = headerHt.entrySet().iterator();
		while(it.hasNext())
		{
			Entry entry = (Entry) it.next();
			key = (String)entry.getKey();
			//˼¼id⣬ֶζupdate
			if(!key.equalsIgnoreCase(SystemConstant.DB_COLUMNNAME_RECORDID_MYSQL))
			{
				//
				value = ((String)entry.getValue()).replaceAll("'","''");//mysqlݿʹ"'"ַݵ
				//÷ţֵõ
				newValueString.append("`").append(key).append("`="); //keycolumnName), e.g:`dec`
				newValueString.append("'").append(value).append("',"); //value, e.g: 'value1'
			}
		}		
		//sql
		//whereӾɸfull_pathֶεΪֶϵԼظ¼ܱinsert
		sql.append("update ").append(info.getTableName()).append(" set ");
		sql.append(newValueString.substring(0, newValueString.length()-1));//ȥһ","
		sql.append(" where `").append(SystemConstant.DB_COLUMNNAME_RELATIVEPATH_MYSQL).append("`='");
		sql.append(headerHt.get(SystemConstant.DB_COLUMNNAME_RELATIVEPATH_MYSQL)).append("'; ");

		logger.debug("update sql="+sql);
		return sql.toString();
	}

	
	/**
	 * <PRE> ݿָı
	 * ǰ
	 *   1.theConnectionڵ÷importToDb()н
	 *   2.theStatementڵ÷recursiveFileImport()л
	 * ע
	 *   رtheStatementtheConnection
	 * @param theStatement
	 * @param sqlHistory
	 * @param failedInfoHt
	 * @param successInfoHt
	 * @throws SQLException 
	 * @throws SQLException 
	 */
	private static void doClearToRearchive(DBConnectionInfo info, Statement theStatement) throws SQLException 
	{
			theStatement.execute("truncate table "+info.getTableName());
			logger.debug(" truncated table:"+info.getTableName());		
	}


	/**
	 * compose sql as below:
	 * 		insert into TABLENAME (`col1`,`col2`,...,`coln`) values ('val1','val2',...,'valn') 
	 * 20070415 ӷš
	 * */
	private static String composeInsertSQL(String tableName, Hashtable ht,DBConnectionInfo info) {
		//check null
		if(tableName==null) //should get tablename from it
		{
			logger.debug("--[DbControl][composeInsertSQL]--tableName is null!");
			return null;
		}
		if(ht==null || ht.size()==0) //should form sql from the key-value pairs 
		{
			logger.debug("--[DbControl][composeInsertSQL]--filehead is null!");
			return null;
		}
		//
		StringBuffer columnString = new StringBuffer();
		StringBuffer valueString = new StringBuffer();
		StringBuffer sql = new StringBuffer();
		String key,value;
		
		//concatrate columns string and values string
		Iterator it = ht.entrySet().iterator();
		if(info.getDbType()==0)
		{
			while(it.hasNext())
			{
				Entry entry = (Entry) it.next();
				key = (String)entry.getKey();
				//
				value = ((String)entry.getValue()).replaceAll("'","''");//mysqlݿʹ"'"ַݵ
				//÷ţֵõ
				columnString.append("`").append(key).append("`,"); //keycolumnName), e.g:`id`
				valueString.append("'").append(value).append("',"); //value, e.g: 'value1'
			}
		}
		else if(info.getDbType()==2)
		{
			while(it.hasNext())
			{
				Entry entry = (Entry) it.next();
				key = (String)entry.getKey();
				//
				value = ((String)entry.getValue()).replaceAll("'","''");//mysqlݿʹ"'"ַݵ
				//÷ţֵõ
				if(key.contains("-")||containsSepciedChar(key))
					key="["+key+"]";
				columnString.append(key).append(","); //keycolumnName), e.g:`id`
				valueString.append("'").append(value).append("',"); //value, e.g: 'value1'
			}
		}
		
		//generate sql
		sql.append("insert into ").append(tableName).append(" (");
		sql.append(columnString.substring(0,columnString.length()-1));
		sql.append(") values (");//each column is set to be "varchar" type
		sql.append(valueString.substring(0,valueString.length()-1));
		sql.append(")");
		//for test
//		String encodedSQL = null;
//		try {
//			encodedSQL = new String(sql.toString().getBytes("gbk"),"utf8");
//			logger.debug("#####@@@@@@@@@@@@@@@ ENCODED: %%%%%%%%%%%^^^^^^^^^^\n"+encodedSQL);
//		} catch (UnsupportedEncodingException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		} 
		
//		return encodedSQL;//sql.toString();
		return sql.toString();
	}

	/**
	 * 20070429ӷҳ
	 * */
	public static void queryTableRecords(DBConnectionInfo connInfo, int start, int rows,ArrayList colList,ArrayList recordList) throws ClassNotFoundException, InstantiationException, IllegalAccessException, SQLException, InstantiationException {
	
		Statement theQueryStatement = null;
		ResultSet rs = null;
		String sql = "";
		Hashtable record=null;
		logger.debug("--------[DbControl][queryTableRecords]--in");

		try {
			DbControl.getConnection(connInfo);	//open a reuseable connection
			logger.debug("----Got connection.Begin to execute sql");

			//use(reuse)the statement to execute sql
			theQueryStatement = DbControl.getConnection(connInfo).createStatement();
			//change database
			sql = "use "+connInfo.getDatabase();
	        theQueryStatement.executeUpdate(sql);
	        String firstColumn="";
	        if(connInfo.getDbType()==2)
	        {
	        	sql="SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS  WHERE TABLE_NAME='"+connInfo.getTableName()+"'";
	        	rs = theQueryStatement.executeQuery(sql);
	        	if(rs.next())
	        	{
	        		firstColumn=rs.getString("COLUMN_NAME");
	        	}
	        	
	        	
	        }
	        //execute query
	        switch(connInfo.getDbType())
	        {
	        	case 0:
	        		sql = "select * from "+connInfo.getTableName() +" limit "+start+","+rows; //startȡrows
	        		break;
	        	case 2:
	        		sql="select top "+rows+" * from "+connInfo.getUserName()+"."+connInfo.getTableName()+" where ("+firstColumn+" NOT IN (SELECT TOP "+ rows*(start/rows)+" "+firstColumn+"  FROM "+connInfo.getUserName()+"."+connInfo.getTableName()+"))";
	        		break;
	        	default:
	        		sql = "select * from "+connInfo.getTableName() +" limit "+start+","+rows; //startȡrows
	        	break;
	        		
	        }
	        
	        rs = theQueryStatement.executeQuery(sql);
	        String colName = "";
			ResultSetMetaData rsmd;
			try {
				rsmd = rs.getMetaData();
				for(int i=1; i<=rsmd.getColumnCount(); i++) //begin from 1?
				{
					colName = rsmd.getColumnName(i);//(ȡֵ)
//					logger.debug("--------[DbControl][getTableColumnName]--rsmd:columnCount="+i+" "+colName);
					colList.add(colName);
				}
			} catch (SQLException e) {
				throw new SQLException(e.getMessage());
			}  
			while(rs.next()) //rsеÿһ¼
			{
//				logger.debug("--[DbControl][resultSetToRecordList] -- new record #"+recordNo);
				record = new Hashtable();//ÿΪһµļ¼Ӧnewһ
				//rsеĸ¼ֵֶHashtable͵Ķ
				for(int i=1; i<=rsmd.getColumnCount(); i++) //begin from 1
				{
					//record.put(""+i, rs.getObject(i)==null?"":rs.getObject(i)); //keyvalue кţֵ
					colName=rsmd.getColumnName(i);
					Object temp=rs.getObject(colName);
					record.put(""+i, temp==null?"":temp); //keyvalue кţֵ
//					logger.debug("--[DbControl][resultSetToRecordList] -- column:"+i+"--"+rs.getObject(i));
				}
				//иֵֶļ¼recordղŵбrecordList
				recordList.add(record);
			}

		} finally
		{
			DbControl.closeAll();
		}

		
	}
	public static void queryTableRecords(DBConnectionInfo connInfo, int start, int rows,ArrayList recordList) throws ClassNotFoundException, InstantiationException, IllegalAccessException, SQLException, InstantiationException {
		
		Statement theQueryStatement = null;
		ResultSet rs = null;
		String sql = "";
		Hashtable record=null;
		logger.debug("--------[DbControl][queryTableRecords]--in");

		try {
			DbControl.getConnection(connInfo);	//open a reuseable connection
			logger.debug("----Got connection.Begin to execute sql");

			//use(reuse)the statement to execute sql
			theQueryStatement = DbControl.getConnection(connInfo).createStatement();
			//change database
			sql = "use "+connInfo.getDatabase();
	        theQueryStatement.executeUpdate(sql);
	        //execute query
	        switch(connInfo.getDbType())
	        {
	        case 0:
	        	 sql = "select * from "+connInfo.getTableName() +" limit "+start+","+rows; //startȡrows
	        	 break;
	        case 2:
	        	sql="select top "+rows+" * from "+connInfo.getUserName()+"."+connInfo.getTableName()+" where (record_Id NOT IN (SELECT TOP "+ rows*(start/rows)+"  record_Id FROM "+connInfo.getUserName()+"."+connInfo.getTableName()+"))";
        		break;
	        default:
	        	sql = "select * from "+connInfo.getTableName() +" limit "+start+","+rows; //startȡrows
	        	 break;
	        	
	        }
	       
	        rs = theQueryStatement.executeQuery(sql);
	        String colName = "";
			ResultSetMetaData rsmd;
			try {
				rsmd = rs.getMetaData();
				
			} catch (SQLException e) {
				throw new SQLException(e.getMessage());
			}  
			while(rs.next()) //rsеÿһ¼
			{
//				logger.debug("--[DbControl][resultSetToRecordList] -- new record #"+recordNo);
				record = new Hashtable();//ÿΪһµļ¼Ӧnewһ
				//rsеĸ¼ֵֶHashtable͵Ķ
				for(int i=1; i<=rsmd.getColumnCount(); i++) //begin from 1
				{
					colName=rsmd.getColumnName(i);
					Object temp=rs.getObject(colName);
					record.put(""+i, temp==null?"":temp); //keyvalue кţֵ
//					logger.debug("--[DbControl][resultSetToRecordList] -- column:"+i+"--"+rs.getObject(i));
				}
				//иֵֶļ¼recordղŵбrecordList
				recordList.add(record);
			}

		} finally
		{
			DbControl.closeAll();
		}

		
	}
	
	/** ҳܣѯȫ¼ */
	public static ArrayList queryTableRecords(DBConnectionInfo connInfo) throws ClassNotFoundException, InstantiationException, IllegalAccessException, SQLException, InstantiationException {
		
		Statement theQueryStatement = null;
		ResultSet rs = null;
		ArrayList colList = new ArrayList();
		ArrayList recordList=new ArrayList();
		Hashtable record=null;
		String sql = "";
		logger.debug("--------[DbControl][queryTableRecords]--in");

		try {
			DbControl.getConnection(connInfo);	//open a reuseable connection
			logger.debug("----Got connection.Begin to execute sql");

			//use(reuse)the statement to execute sql
			theQueryStatement = DbControl.getConnection(connInfo).createStatement();
			//change database
			sql = "use "+connInfo.getDatabase();
	        theQueryStatement.executeUpdate(sql);
	        //execute query
	        sql = "select * from "+connInfo.getTableName(); //
	        rs = theQueryStatement.executeQuery(sql);
			String colName = "";
			ResultSetMetaData rsmd;
			try {
				rsmd = rs.getMetaData();
				for(int i=1; i<=rsmd.getColumnCount(); i++) //begin from 1?
				{
					colName = rsmd.getColumnName(i);//(ȡֵ)
//					logger.debug("--------[DbControl][getTableColumnName]--rsmd:columnCount="+i+" "+colName);
					colList.add(colName);
				}
			} catch (SQLException e) {
				throw new SQLException(e.getMessage());
			}   
			while(rs.next()) //rsеÿһ¼
			{
//				logger.debug("--[DbControl][resultSetToRecordList] -- new record #"+recordNo);
				record = new Hashtable();//ÿΪһµļ¼Ӧnewһ
				//rsеĸ¼ֵֶHashtable͵Ķ
				for(int i=1; i<=rsmd.getColumnCount(); i++) //begin from 1
				{
					record.put(""+i, rs.getObject(i)==null?"":rs.getObject(i)); //keyvalue кţֵ
//					logger.debug("--[DbControl][resultSetToRecordList] -- column:"+i+"--"+rs.getObject(i));
				}
				//иֵֶļ¼recordղŵбrecordList
				recordList.add(record);
			}

		} finally
		{
			DbControl.closeAll();
		}

		return colList;
	}

	public static List getTableColumnName(ResultSet rs) throws SQLException {
		List colList = new ArrayList();
		String colName = "";
		ResultSetMetaData rsmd;
		try {
			rsmd = rs.getMetaData();
			for(int i=1; i<=rsmd.getColumnCount(); i++) //begin from 1?
			{
				colName = rsmd.getColumnName(i);//(ȡֵ)
//				logger.debug("--------[DbControl][getTableColumnName]--rsmd:columnCount="+i+" "+colName);
				colList.add(colName);
			}
		} catch (SQLException e) {
			throw new SQLException(e.getMessage());
		}   
		
		return colList;
	}

	
	/**
	 * ResultSet -> List(of record)
	 * ÿrecordHashtableԱֶϢ
	 * */
	public static List resultSetToRecordList(ResultSet rs) throws SQLException {

		List recordList = new ArrayList(); //øм¼бÿԪؽǸHashtable͵ĶΪһ¼
	 	Hashtable record;//һ¼ÿ¼ֶ(вȷ)ʹHashtableøֶεkey-valueӳ
    	ResultSetMetaData rsmd; //ȡиϢ
    	
		if(rs!=null)
		{
			rsmd = rs.getMetaData();
			int columnCount = rsmd.getColumnCount(); //
			int recordNo = 1;
			while(rs.next()) //rsеÿһ¼
			{
//				logger.debug("--[DbControl][resultSetToRecordList] -- new record #"+recordNo);
				recordNo++;
				record = new Hashtable();//ÿΪһµļ¼Ӧnewһ
				//rsеĸ¼ֵֶHashtable͵Ķ
				for(int i=1; i<=columnCount; i++) //begin from 1
				{
					record.put(""+i, rs.getObject(i)==null?"":rs.getObject(i)); //keyvalue кţֵ
//					logger.debug("--[DbControl][resultSetToRecordList] -- column:"+i+"--"+rs.getObject(i));
				}
				//иֵֶļ¼recordղŵбrecordList
				recordList.add(record);
			}
		}

    	return recordList;
	}

	
	/**
	 * ɽʱõ2ĬֶΣɵlist
	 * */
	public static List getDefaultColumnEntryList() {
		// TODO Auto-generated method stub
		List defaultColumnList = new ArrayList();
		
		//add 2 TableColumnBeans into the Hashtable
		//add record id(primary key)
		TableColumnBean tcbId = new TableColumnBean();
		tcbId.setColumnName(SystemConstant.DB_COLUMNNAME_RECORDID_MYSQL); //"record_Id"
		tcbId.setType(SystemConstant.DB_DATATYPE_MYSQL_RECORDID);  //bigint
		tcbId.setLength(SystemConstant.DB_COLUMNLENGTH_RECORDID); //20
		tcbId.setComment(SystemConstant.MSG_UI_TABLEVIEW_COLUMNCOMMENT_ID);//"Primary key"
		defaultColumnList.add(tcbId);
	
		//relative path(text/varchar)
		TableColumnBean tcbPath = new TableColumnBean();
		tcbPath.setColumnName(SystemConstant.DB_COLUMNNAME_RELATIVEPATH_MYSQL); //"path"(relative path)
		tcbPath.setType(SystemConstant.DB_DATATYPE_MYSQL_RELATIVEPATH);  // text/varchar
		tcbPath.setLength(SystemConstant.DB_COLUMNLENGTH_RELATIVEPATH);  //65535 for "text", 255 for "varchar"
		tcbPath.setComment(SystemConstant.MSG_UI_TABLEVIEW_COLUMNCOMMENT_PATH);//"unique key"
		defaultColumnList.add(tcbPath);
				
		return defaultColumnList;
	}

	public static int queryTotalCount(DBConnectionInfo connInfo) throws SQLException, ClassNotFoundException, InstantiationException, IllegalAccessException {
		Statement theQueryStatement = null;
		ResultSet rs = null;
		String sql = "";
		int count = 0;

		try {
			DbControl.getConnection(connInfo);	//open a reuseable connection
			logger.debug("----Got connection.Begin to execute sql");

			//use(reuse)the statement to execute sql
			theQueryStatement = DbControl.getConnection(connInfo).createStatement();
			//change database
			sql = "use "+connInfo.getDatabase();
	        theQueryStatement.executeUpdate(sql);
	        //execute query
	        sql = "select count(*) from "+connInfo.getTableName(); //
	        rs = theQueryStatement.executeQuery(sql);
	        if(rs.next())
	        	count = (int) rs.getInt(1); 

		} finally
		{
			DbControl.closeAll();
		}

		return count;
	}


	

} 
