package org.cvo.fithas.util;

import java.io.IOException;

import org.cvo.fithas.dbsupport.DBConnectionInfo;
import org.cvo.fithas.exception.NotSpecifiedInfoException;

public class DBConfHelper {
    
	static org.cvo.fithas.util.Logger logger = new org.cvo.fithas.util.Logger(DBConfHelper.class.getName () ) ;
	
	private static Config config = null;
	
	
	/** HARD CODE HERE.
	 * Read config information from the properties file, named "DBSetting.properties".
	 * Singleton, only one config of DBSetting exists.
	 * @throws IOException 
	 * */
    public static Config getDBConf() throws IOException{
    	if(config==null)
    	{
    		/****************************************************
    		 * for standalone jar, use "/conf/DBSetting.properties";
    		 * for program within eclipse, use "DBSetting.properties"
    		 *****************************************************/
    		config = new Config(SystemConstant.PROPFILE_CONFIG_DB); //the properties file name 
    	}
        return config;
    }

    /**
     * Get an attribute from the config.
     * @throws IOException 
     * */
    public static String getAttribute(String key) throws IOException
    {
    	if(config==null)
    		getDBConf();
    	return config.getAttribute(key);
    }
    
    
    
    /**
     * Fetch all connection info from config file, and set them in DBConnectionInfo.
     * Use MySQL config info as default.
     * 
     * @return DBConnectionInfo
     * @throws IOException 
     * */
    private static DBConnectionInfo getDefaultDBConnInfo() throws IOException
    {
    	return getDefaultDBConnInfo(SystemConstant.DB_TYPE_MYSQL);
    }
    
    
    /**
     * Fetch all connection info from config file, and set them in DBConnectionInfo.
     * 
     * @return DBConnectionInfo
     * @throws IOException 
     * */
    private static DBConnectionInfo getDefaultDBConnInfo(int dbType) throws IOException
    {
    	if(config==null)
    		getDBConf();
    	DBConnectionInfo info = null;
    	if(dbType==SystemConstant.DB_TYPE_MYSQL)
    	{
    		info = new DBConnectionInfo(); //use MySQL as default
	    	info.setDriver(getAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_DRIVER));
	    	info.setServer(getAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_SERVER));
	    	info.setPort(getAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_PORT));
	    	info.setDatabase(getAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_DATABASE));
	    	info.setUserName(getAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_USERNAME));
	    	info.setPassword(getAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_PASSWORD));
    	}
    	else if(dbType==SystemConstant.DB_TYPE_ORACLE)
    	{logger.debug("Unsupport database type!");}
    	else if(dbType==SystemConstant.DB_TYPE_SQLSERVER)
    	{logger.debug("Unsupport database type!");}
    	return info;
    }
    
    
    
    /**
     * update an item, and write back to file
     * @throws NotSpecifiedInfoException 
     * */
    public static void saveAttribute(DBConnectionInfo info) throws IOException, NotSpecifiedInfoException
    {
    	if(config==null)
    		getDBConf();
    	//update property item
    	if(info==null)
    	{
    		throw new NotSpecifiedInfoException();
    	}
    	if(info.getDbType()==SystemConstant.DB_TYPE_MYSQL)
    	{
    		//set property item: "db.mysql.type=0"
    		//config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_TYPE, ""+SystemConstant.DB_TYPE_MYSQL);
    		config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_SERVER, info.getServer()); //server
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_PORT, info.getPort()); //server
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_USERNAME, info.getUserName()); //username
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_PASSWORD, info.getPassword()); //password
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_DATABASE, info.getDatabase()); //database
    	}
    	
    	if(info.getDbType()==SystemConstant.DB_TYPE_ORACLE)
    	{
    		//set property item: "db.mysql.type=0"
    		//config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_TYPE, ""+SystemConstant.DB_TYPE_MYSQL);
    		config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_ORACLE_SERVER, info.getServer()); //server
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_ORACLE_PORT, info.getPort()); //server
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_ORACLE_USERNAME, info.getUserName()); //username
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_ORACLE_PASSWORD, info.getPassword()); //password
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_ORACLE_DATABASE, info.getDatabase()); //database
    	}
    	if(info.getDbType()==SystemConstant.DB_TYPE_SQLSERVER)
    	{
    		//set property item: "db.mysql.type=0"
    		//config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_MYSQL_TYPE, ""+SystemConstant.DB_TYPE_MYSQL);
    		config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_SQL_SERVER, info.getServer()); //server
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_SQL_PORT, info.getPort()); //server
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_SQL_USERNAME, info.getUserName()); //username
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_SQL_PASSWORD, info.getPassword()); //password
        	config.setAttribute(SystemConstant.CONFIG_DB_PROPKEY_SQL_DATABASE, info.getDatabase()); //database
    	}
    	
    	//write back to file
    	config.writeConf(SystemConstant.PROPFILE_CONFIG_DB, SystemConstant.CONFIG_DB_PROPFILE_COMMENT);//the properties file name
    }
    
    
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		Config conf;
		try {
			conf = DBConfHelper.getDBConf();
			conf.print();
			DBConnectionInfo info = getDefaultDBConnInfo();
			info.print();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		

	}

}
