package org.cvo.fithas.util;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cvo.fithas.fitsfile.GenericHeadAnalysis;

public class Utils {
	static org.cvo.fithas.util.Logger logger = new org.cvo.fithas.util.Logger(Utils.class.getName () ) ;
	
	/**
	 * Convert Hashtable to List.
	 * Add all values in Hashtable(not the key-value entries) to the list.
	 * if the param is null, just return a list whose size is 0.
	 * @param ht Hashtable
	 * @return List
	 * */
	public static List HashtableToList(Hashtable ht)
	{
		logger.debug("~~~~~[Utils][HashtableToList]~~~");
		List list = new ArrayList();
		if(ht!=null)
		{
			//add the values collection(in the order as its iterator),to the end of the list
			list.addAll(ht.values()); 
		}
		return list;
	}

	
	public static void printCollection(Object obj)
	{
		if (obj instanceof Hashtable) {
			logger.debug("~~~~~[Utils][printCollection]~~~obj instanceof Hashtable");
			Hashtable ht = (Hashtable) obj;
			Enumeration enums = ht.keys(); //values
			while(enums.hasMoreElements())//
			{
				Object o = enums.nextElement();
				logger.debug(o+"-"+ht.get(o).toString());				
			}
		}
		else if (obj instanceof List) {
			logger.debug("~~~~~[Utils][printCollection]~~~obj instanceof List");
			List list = (List) obj;
			for(Iterator it = list.iterator(); it.hasNext(); )
			{
				Object o = it.next();
				logger.debug(o);				
			}
		}
		else
		{
			logger.debug("~~~~~[Utils][printCollection]~~~obj unknown type");
		}
	}
	

	/**
	 * Concatenate String Array with certain separator char.
	 * @param strArray -- Strings to be concatenated
	 * @param separator String--"," or ";" or any char to separate Strings, but please don't use those in the String array
	 * @return String
	 */
	public static String concatString(String[] strArray) {	
		return concatString(strArray, SystemConstant.UI_FILE_SEPERATOR); 
	}
	

	/**
	 * Concatenate String Array with certain separator char.
	 * @param strArray -- Strings to be concatenated
	 * @param separator String--"," or ";" or any char to separate Strings, but please don't use those in the String array
	 * @return String
	 */
	public static String concatString(String[] strArray, String separator) {	
		
		StringBuffer result = new StringBuffer();
		
		//check null
		if(strArray==null || strArray.length==0)
			return "";
		if(separator==null)
			separator = SystemConstant.UI_FILE_SEPERATOR; //set default
		
		//else, concatenate
		result.append(strArray[0]);
		for (int i = 1; i < strArray.length; i++)
		{
			result.append(separator).append(strArray[i]);
		}
		return result.toString();
	}
	
	
	
	
	/**
	 * Append a common String part to all elements in an Array.
	 * Mainly used in : array_of_file_name + files_common_dir => array_of_file_full_path 
	 * @param strArray
	 * @param strToAppend
	 * @param isAddToHead
	 * @return
	 */
	public static String[] appendCommonStringToAll(String[] strArray, String strToAppend, boolean isAddToHead)
	{
		if(strArray!=null)
		{
			if(isAddToHead)
			{
				for(int i=0; i<strArray.length; i++)
				{
					strArray[i] = strToAppend + strArray[i]; //add the common part before the original string
				}
			}
			else
			{
				for(int i=0; i<strArray.length; i++)
				{
					strArray[i] = strArray[i] + strToAppend; //add the common part after the original string
				}				
			}
		}
		return strArray;
	}
	
	
	/**
	 * @param fileName 
	 * 			String : ļȫ·ַ"D:/data/Bk672.fits""D:\\data\\samples\\Bk672.fits"
	 * @return 
	 * 			String:ȡõļĿ¼Ĳ(Լĩβб)
	 * 			
	 */
	public static String getFilePathPart(String fileName)
	{
		if(fileName==null)
			return null;
		File f = new File(fileName);
		return f.getParent()+File.separator;
		
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {

		String name = "D:/data/samples/Bk672.fits"; 
		File f = new File(name);
		try {
			System.out.println("f.getAbsolutePath()="+f.getAbsolutePath());
			System.out.println("f.getCanonicalPath()="+f.getCanonicalPath());
			System.out.println("f.getName()="+f.getName());
			System.out.println("f.getParent()="+f.getParent());
			System.out.println("f.getPath()="+f.getPath());
			System.out.println("f.getAbsoluteFile()="+f.getAbsoluteFile());
			System.out.println("f.getCanonicalFile="+f.getCanonicalFile());
			System.out.println("f.getClass()="+f.getClass());
			System.out.println("f.getParentFile()="+f.getParentFile());
			System.out.println("f.toURI()="+f.toURI());
			System.out.println("f.toURL()="+f.toURL());
			System.out.println("f.pathSeparator="+f.pathSeparator); //4ʵʹstaticʹ÷File.pathSeparator
			System.out.println("f.separator="+f.separator);
			System.out.println("f.pathSeparatorChar="+f.pathSeparatorChar);
			System.out.println("f.separatorChar="+f.separatorChar);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		

	}

	/**
	 * ļǷָģʽ
	 * ʹjavaʽַƥ䣬ִСд
	 * ָģʽΪnullδָĬΪƥ䡣
	 * */
	public static boolean checkFileNamePattern(String fullPath, String filter) {
		 //ļʽƥ͹
		File f = new File(fullPath);
	   	 boolean isPatternOk;
	   	 if(filter!=null)
	   	 {
		   	 //һʹòִСдģʽ(ǰ"(?i)")
	   		 filter = "(?i)" +filter;
	   		 Pattern p = Pattern.compile(filter);
	       	 Matcher m = p.matcher(f.getName()); //ֻļ·
	       	 isPatternOk = m.matches();//find();//matches();
	       	 GenericHeadAnalysis.logger.debug("fileName="+f.getName()+"\nfilter="+filter+"\nm.matches()="+isPatternOk);
	   	 }
	   	 else
	   	 {
	   		 isPatternOk = true;
	   	 }
	   	 //ֻƥ˵ĲŽһĽ
	   	 
		return isPatternOk;
	}

	
	
	/** ûַͨתΪΪʽĶӦַ
	 *  Ҫ滻javaʽеıַ +*?.-\()[]{}
	 * */
	public static String StrToRegexp(String text) {
		String result = null;
		
		if(text==null) //ֵ
			return null;
		if(text.equals(""))
			return "";

		/*   \ -> \\   
		 * 滻ע⣺replaceAll(String,String)ĵһǰreg﷨
		 * 
		 * ʵҪ \ ֱַindexOf("\\")
		 * 滻ʱʹreplaceAll()Ҫ"\\\\"Ϊregexp
		 * 
		 * Կʹã text.replaceAll("\\\\", "somestr");
		 * ʹãtext.replaceAll("\\", "somestr");
		 * 
		 * Ҳʹãtext.replaceAll("\\\\", "\\"); 
		 * ᱨjava.lang.StringIndexOutOfBoundsException: String index out of range: 1
		 * 
		 * ܴﵽ,\\\ǣ
		 * text.replaceAll("\\\\", "\\\\\\\\") 
		 * еһʱѾ\\ˣΪʽһб
		 * еڶʱѾ\\\\ʽΪַԭStringУʵ\\
		 * ֮Ҫ \\ javaдַʾӦ"\\\\"ô "\\\\" 4ַôòreplaceAll()أ-8б
		 * */
		logger.debug("before change  \\ to \\\\ :"+text);
		logger.debug(" \\ is at: "+text.indexOf("\\"));
		logger.debug("\\\\ is at: "+text.indexOf("\\\\"));
		text = text.replaceAll("\\\\", "\\\\\\\\"); 
		logger.debug("after change \\ to \\\\ :"+text);
		
		//滻
		String[] specialChars = {".", "+", "*", "?", "-", "(", ")", "[", "]", "{", "}", "^", "$", "|"};
		for(int i=0; i<specialChars.length; i++)
		{
			/* 磺   . -> \.    		 
			 * ܴﵽǣ text.replaceAll("\\.", "\\\\.") 
			 * еһʱѾ\.ˣΪʽһַͨ㣬"ַ"
			 * еڶʱѾ\\.ʽΪַԭStringУʵ\.
			 * ֮Ҫ \. javaдַʾӦ"\\."ô "\\." 3ַôòreplaceAll()أ-4бһ
			 * */		
			logger.debug("before replace char: "+specialChars[i]+" to \\"+specialChars[i]+" ---text="+text);
			text = text.replaceAll("\\"+specialChars[i], "\\\\"+specialChars[i]); //磺 \\. -> \\\\.
			logger.debug("after replace char: "+specialChars[i]+" to \\"+specialChars[i]+" ---text="+text);
		}
		
		result = text;
		return result;
	}


}
